--[[
	This code initially created by Tekkub for GnomishVendorShrinker. Heavily modified at this point, but he still gets credit for the core UI.
	MonoType.ttf is the free Monospace Typewriter font created by Manfred Klein, available here: http://www.dafont.com/monospace-typewrite.font
]]--
local name, addon = ...;
local L = addon.L;
local GetMountStats = addon.GetMountStats
local IsCritterUsable = addon.IsCritterUsable

local NUMROWS, ROWHEIGHT, ICONSIZE, GAP, SCROLLSTEP, ROTATETIME = 15, 22, 17, 4, 5, 6
local COMPANIONTYPE = "CRITTER"
local ALTSORT = false -- Boolean; set to true if mounts should be sorted by speed rather than name
local MOUNTSSORTED = {} -- Array of mounts sorted by speed. Rebuilt every time ALTSORT is set to true
local pq = {} -- Just a temp array

local function RebuildMountsSorted()
	for i=1,GetNumCompanions("MOUNT") do -- NO LOCALIZE
		-- creatureID, creatureName, spellID, icon, active
		local info = {GetCompanionInfo("MOUNT", i)}
		local stats = {info, GetMountStats(info[3]), i} -- NO LOCALIZE
		tinsert(pq, stats)
	end
	table.sort(pq, function(a,b)
		return a[2][1]>b[2][1] or (a[2][1]==b[2][1] and a[1][2]<b[1][2]) -- Sort by speed then alpha. God that's ugly.
	end)
	MOUNTSSORTED = pq
	pq = {}
end

-- For some reason, the 12 CompanionButtons on the default UI are protected, which implicitly protects their parent.
for i=1,12 do
	SecureHandlerWrapScript(_G["CompanionButton"..i], "OnShow", _G["CompanionButton"..i],
		[[ self:Hide() self:GetParent():Hide() ]])
end

local FUR = CreateFrame("frame", nil, PetPaperDollFrame)
FUR:SetWidth(315) -- Consider a better way to do this
FUR:SetHeight(NUMROWS*ROWHEIGHT)
FUR:SetPoint("TOPLEFT", 21, -77)
FUR:SetScript("OnEvent", function(self, event, ...) if self[event] then return self[event](self, event, ...) end end)
FUR:Hide()

local FUR3DModel = CreateFrame("PlayerModel")
FUR3DModel:SetFrameStrata("TOOLTIP")
FUR3DModel:SetBackdrop({
	bgFile = "Interface/Tooltips/UI-Tooltip-Background",
	edgeFile = "Interface/Tooltips/UI-Tooltip-Border",
	tile = nil, tileSize = 0, edgeSize = 10,
	insets = { left = 3, right = 3, top = 3, bottom = 3 }
})
FUR3DModel:SetBackdropColor(0.3, 0.3, 0.3 ,1)
FUR3DModel:Hide()
FUR3DModel:SetScript("OnShow", function(self) self:SetFacing(0) end)
FUR3DModel:SetScript("OnUpdate", function(self, e)
	self:SetFacing(self:GetFacing() + e*math.pi/ROTATETIME)
end)

-- row.onClick
local function OnClick(self, button)
	if IsModifiedClick("CHATLINK") then
		local spellId = select(3, GetCompanionInfo(COMPANIONTYPE, self:GetID()))
		ChatEdit_InsertLink(GetSpellLink(spellId));
	-- elseif IsModifiedClick("DRESSUP") then 
		-- if not(DressUpFrame:IsShown()) then 
			-- ShowUIPanel(DressUpFrame); 
		-- end 
		-- DressUpModel:SetCreature(GetCompanionInfo(COMPANIONTYPE,self:GetID())) 
	elseif not IsFlying() then -- Protect against aerial dismounts
		CancelShapeshiftForm()
		CallCompanion(COMPANIONTYPE,self:GetID())
	end
end

-- "currency icon".onEnter
local function OnEnter(self)
	GameTooltip:SetOwner(self, "ANCHOR_RIGHT")
	if self.mouseover then
		GameTooltip:SetText(self.mouseover)
	end
end

-- "currency icon".onLeave
local function OnLeave()
	GameTooltip:Hide()
	ResetCursor()
end

local function SetValue(self, text, icon)
	self.text:SetText(text)
	self.icon:SetTexture(icon)
	
	self.icon:SetPoint("RIGHT")
	self.text:SetPoint("RIGHT", self.icon, "LEFT", -GAP/2, 0)
	self:Show()
end

-- Return an "alt currency frame"; create it if there are no unused ones on this row.
local function GetAltCurrencyFrame(frame)
	for i,v in ipairs(frame.altframes) do
		if not v:IsShown() then return v end
	end

	--local anchor = #frame.altframes > 0 and frame.altframes[#frame.altframes].text or frame.MountInfo
	local anchor = frame.MountInfo
	local oldPoint = {anchor:GetPoint(1)} -- This shoud be the attachment to the right side of the row
	
	local f = CreateFrame('Frame', nil, frame)
	f:SetWidth(ICONSIZE)
	f:SetHeight(ICONSIZE)
	
	--f:SetPoint("RIGHT", anchor, "LEFT")
	
	f.icon = f:CreateTexture()
	f.icon:SetWidth(ICONSIZE)
	f.icon:SetHeight(ICONSIZE)

	f.text = f:CreateFontString(nil, nil, "NumberFontNormalSmall")

	anchor:ClearAllPoints()
	f:SetPoint(unpack(oldPoint))
	anchor:SetPoint("RIGHT", f.icon, "LEFT", -4, 0)

	f.SetValue = SetValue

	f:EnableMouse(true)
	f:SetScript("OnEnter", OnEnter)
	f:SetScript("OnLeave", OnLeave)

	table.insert(frame.altframes, f)
	return f

end


local function AddAltCurrency(frame, i)
	local lastframe = frame.MountInfo

	do 
		local f = frame:GetAltCurrencyFrame()
		local texture, mouseover
		if frame.stats[5] == 1 then -- Is single passenger mount
			texture = "Interface\\Icons\\INV_Misc_GroupLooking"
			mouseover = format(L["%d Passengers"], frame.stats[5])
		elseif frame.stats[5] > 1 then -- Is multi-passenger mount
			texture = "Interface\\Icons\\INV_Misc_GroupNeedMore"
			mouseover = format(L["%d Passengers"], frame.stats[5])
		elseif frame.stats[4] then -- Is a water mount
			texture = "Interface\\Icons\\Ability_Druid_AquaticForm"
			mouseover = L["Aquatic Mount"]
		elseif frame.stats[3] then -- Is a flight mount
			texture = "Interface\\Icons\\Ability_Druid_FlightForm"
			mouseover = L["Flying Mount"]
		elseif frame.stats[2] then -- Is a land mount
			texture = "Interface\\Icons\\Ability_Druid_TravelForm"
			mouseover = L["Land Mount"]
		else
			print("what is this I don't even")
		end
		f:SetValue("", texture)
		f.index, f.mouseover = i, mouseover
		f:Show()
		lastframe = f.icon
	end
	
	-- SAVE: Might want to play with this if we get flying passengers in Cataclysm
	-- if frame.stats[5] > 0 then -- It's a passenger mount
	----	print("Passenger mount found at mount# "..frame:GetID())
		-- local texture, mouseover
		-- if frame.stats[5] == 1 then
			-- texture = "Interface\\Icons\\INV_Misc_GroupLooking"
		-- elseif frame.stats[5] > 1 then
			-- texture = "Interface\\Icons\\INV_Misc_GroupNeedMore"
		-- end
		-- mouseover = string.format(PASSENGER_MOUNT,frame.stats[5])
		-- local f = frame:GetAltCurrencyFrame()
	----	f:SetValue(frame.stats[5], texture)
		-- f:SetValue("", texture)
		-- f.index, f.mouseover = i, mouseover
		-- f:Show()
		-- lastframe = f.icon
	-- end
	frame.MountInfo:SetPoint("RIGHT", lastframe, "LEFT", -GAP, 0)
end

local RowOnEnter = function(self)
	GameTooltip:SetOwner(self, "ANCHOR_RIGHT")
	local spell = select(3, GetCompanionInfo(COMPANIONTYPE, self:GetID()))
	GameTooltip:SetHyperlink(GetSpellLink(spell))
	GameTooltip:Show()
	FUR3DModel:SetPoint("TOPLEFT",GameTooltip,"BOTTOMLEFT")
	FUR3DModel:SetPoint("TOPRIGHT",GameTooltip,"BOTTOMRIGHT")
	 local size = GameTooltip:GetWidth()
	FUR3DModel:SetPoint("BOTTOMLEFT",GameTooltip,"BOTTOMLEFT", 0, -size/2)
	FUR3DModel:SetPoint("BOTTOMRIGHT",GameTooltip,"BOTTOMRIGHT", 0, -size/2)
	-- local size = GameTooltip:GetWidth()
	-- FUR3DModel:SetWidth(size)
	-- FUR3DModel:SetHeight(size)
	FUR3DModel:SetCreature(GetCompanionInfo(COMPANIONTYPE,self:GetID()))
	FUR3DModel:Show()
--	if IsModifiedClick("DRESSUP") then ShowInspectCursor() else ResetCursor() end
end

local RowOnLeave = function()
	GameTooltip:Hide()
	FUR3DModel:Hide()
	ResetCursor()
end

-- Build the rows
local rows = {}
for i=1,NUMROWS do
	local row = CreateFrame('Button', "FURROW"..i, FUR) -- base frame
	row:SetHeight(ROWHEIGHT)
	row:SetPoint("TOP", i == 1 and FUR or rows[i-1], i == 1 and "TOP" or "BOTTOM")
	row:SetPoint("LEFT")
	row:SetPoint("RIGHT", -19, 0)

	row:SetHighlightTexture("Interface\\HelpFrame\\HelpFrameButton-Highlight")
	row:GetHighlightTexture():SetTexCoord(0, 1, 0, 0.578125)

	row:SetScript('OnClick', OnClick)
	row:SetScript('OnDragStart', function(self, button)
		PickupCompanion(COMPANIONTYPE, self:GetID())
	end)
	row:RegisterForDrag("LeftButton","RightButton")
	
	local backdrop = row:CreateTexture(nil, "BACKGROUND")
	backdrop:SetAllPoints()
	backdrop:SetTexture("Interface\\Tooltips\\UI-Tooltip-Background")
	row.backdrop = backdrop

	local mountIndex = row:CreateFontString()
	mountIndex:SetFont("Interface\\AddOns\\CRAP\\MonoType.ttf",11,"MONOCHROME")
	mountIndex:SetTextColor(1.0,0.82,0)
	mountIndex:SetText("888"); local width = mountIndex:GetStringWidth()
	mountIndex:SetText("")
	mountIndex:SetWidth(width)
	mountIndex:SetPoint("LEFT", row, "LEFT", 2, 0)
	mountIndex:SetJustifyH("RIGHT")
	row.mountIndex = mountIndex
	
	local icon = CreateFrame('Frame', nil, row)
	icon:SetHeight(ICONSIZE)
	icon:SetWidth(ICONSIZE)
	icon:SetPoint('LEFT', mountIndex, "RIGHT", 4, 0)

	row.icon = icon:CreateTexture(nil, "BORDER")
	row.icon:SetAllPoints()
	
	local CompanionName = row:CreateFontString(nil, nil, "GameFontNormalSmall")
	CompanionName:SetPoint('LEFT', icon, "RIGHT", GAP, 0)
	CompanionName:SetJustifyH('LEFT')
	row.CompanionName = CompanionName

	local MountInfo = row:CreateFontString(nil, nil, "NumberFontNormalSmall")
	MountInfo:SetPoint('RIGHT', row, "RIGHT", -2, 0)
	row.MountInfo = MountInfo

	row.altframes = {}
	row.AddAltCurrency, row.GetAltCurrencyFrame = AddAltCurrency, GetAltCurrencyFrame

	row:SetScript('OnEnter', RowOnEnter)
	row:SetScript('OnLeave', RowOnLeave)

	rows[i] = row
end


-- Tekkub's trickery. Cached item colors, defaulting to white. Not used atm, but saved.
-- local quality_colors = setmetatable({}, {__index = function() return "|cffffffff" end})
-- for i=1,7 do quality_colors[i] = select(4, GetItemQualityColor(i)) end
local offset = 0
local searchstring
local function Refresh()
	local n = GetNumCompanions(COMPANIONTYPE)
	for i,row in pairs(rows) do
		local j = i + offset
		if j > n then
			row:Hide()
		else
			row.backdrop:Hide()
			
			local creatureID, creatureName, spellID, icon, active, mountIndex
			local isUsable
			if COMPANIONTYPE == "CRITTER" then
				creatureID, creatureName, spellID, icon, active = GetCompanionInfo(COMPANIONTYPE, j)
				isUsable = IsCritterUsable(spellID)
				row:SetID(j)
				mountIndex = j
			else
				if ALTSORT then
					creatureID, creatureName, spellID, icon, active = unpack(MOUNTSSORTED[j][1])
					row.stats =	MOUNTSSORTED[j][2] -- IsCritterUsable = speed, land, flight, water, passengers, isUsableFunc
					row:SetID(MOUNTSSORTED[j][3])
					mountIndex = MOUNTSSORTED[j][3]
				else
					creatureID, creatureName, spellID, icon, active = GetCompanionInfo(COMPANIONTYPE, j)
					row.stats = GetMountStats(spellID) -- IsCritterUsable = speed, land, flight, water, passengers, isUsableFunc
					row:SetID(j)
					mountIndex = j
				end
				isUsable = row.stats[6]()
			end
			
			local aColor = "|cFF80FF60" -- Color for the texts on the line of the active companion
			row.companionID = creatureID
			row.icon:SetTexture(icon)
			if active then
				row.CompanionName:SetText(aColor.. (creatureName or L["<Loading companion data>"]).."|r")
				row.mountIndex:SetText(aColor..mountIndex.."|r")
				row.backdrop:SetGradientAlpha("HORIZONTAL", 0.3,0.6,0.2,0.75, 0.3,0.6,0.2,0)
				row.backdrop:Show()
			else
				row.CompanionName:SetText((creatureName or L["<Loading companion data>"]))
				row.mountIndex:SetText(mountIndex)
			end
			
			if not isUsable then
				row.backdrop:SetGradientAlpha("HORIZONTAL", 1,0,0,0.75, 1,0,0,0)
				row.backdrop:Show()
			end

			row:SetAlpha(searchstring and creatureName and not creatureName:lower():match(searchstring) and 0.5 or 1)
			for i,v in pairs(row.altframes) do v:Hide() end
			if isUsable then
				row.icon:SetVertexColor(1, 1, 1)
			else
				row.icon:SetVertexColor(.9, 0, 0)
			end
			row:Show()
			if COMPANIONTYPE == "MOUNT" then
				-- The "extended cost", AKA icons
				row:AddAltCurrency(j)
				-- The text of the "price", AKA mount speed
				if active then
					row.MountInfo:SetText(aColor..row.stats[1].."%|r")
				else
					row.MountInfo:SetText(row.stats[1].."%")
				end
			else
				row.MountInfo:SetText("")
			end
		end
	end
end

--[[ Search Box ]]--
local editbox = CreateFrame('EditBox', nil, FUR)
editbox:SetAutoFocus(false)
editbox:SetPoint("TOPLEFT", FUR, "BOTTOMLEFT", 8, 2)
editbox:SetWidth(234)
editbox:SetHeight(32)
editbox:SetFrameStrata("HIGH")
editbox:SetFontObject('GameFontHighlightSmall')

local left = editbox:CreateTexture(nil, "BACKGROUND")
left:SetWidth(8) left:SetHeight(20)
left:SetPoint("LEFT", -5, 0)
left:SetTexture("Interface\\Common\\Common-Input-Border")
left:SetTexCoord(0, 0.0625, 0, 0.625)

local right = editbox:CreateTexture(nil, "BACKGROUND")
right:SetWidth(8) right:SetHeight(20)
right:SetPoint("RIGHT", 0, 0)
right:SetTexture("Interface\\Common\\Common-Input-Border")
right:SetTexCoord(0.9375, 1, 0, 0.625)

local center = editbox:CreateTexture(nil, "BACKGROUND")
center:SetHeight(20)
center:SetPoint("RIGHT", right, "LEFT", 0, 0)
center:SetPoint("LEFT", left, "RIGHT", 0, 0)
center:SetTexture("Interface\\Common\\Common-Input-Border")
center:SetTexCoord(0.0625, 0.9375, 0, 0.625)

editbox:SetScript("OnEscapePressed", editbox.ClearFocus)
editbox:SetScript("OnEnterPressed", editbox.ClearFocus)
editbox:SetScript("OnEditFocusGained", function(self)
	if not searchstring then
		self:SetText("")
		self:SetTextColor(1,1,1,1)
	end
end)
editbox:SetScript("OnEditFocusLost", function(self)
	if self:GetText() == "" then
		self:SetText(L["Search..."])
		self:SetTextColor(0.75, 0.75, 0.75, 1)
	end
end)
editbox:SetScript("OnTextChanged", function(self)
	local t = self:GetText()
	searchstring = t ~= "" and t ~= L["Search..."] and t:lower() or nil
	Refresh()
end)
editbox:SetScript("OnShow", function(self)
	self:SetText(L["Search..."])
	self:SetTextColor(0.75, 0.75, 0.75, 1)
end)
-- [[ /Search Box ]]--

local sortbutton = CreateFrame("Button", nil, FUR, "UIPanelButtonTemplate")
sortbutton:SetPoint("TOPRIGHT", FUR, "BOTTOMRIGHT", 8, -4)
sortbutton:SetNormalFontObject("GameFontHighlightSmall")
sortbutton:SetHighlightFontObject("GameFontHighlightSmall")
sortbutton:SetDisabledFontObject("GameFontHighlightSmall")
sortbutton:SetFrameStrata("HIGH")
sortbutton:SetText(L["Sort by speed"]) -- TODO localize
sortbutton:SetHeight(sortbutton:GetTextHeight() + 8)
sortbutton:SetWidth(sortbutton:GetTextWidth() + 8)
sortbutton:SetScript("OnClick", function(self, button)
	ALTSORT = not ALTSORT
	self:SetText(ALTSORT and L["Sort A-Z"] or L["Sort by speed"]) -- TODO localize
	if ALTSORT and #MOUNTSSORTED==0 then -- If MOUNTSSORTED has been built, FUR's event handlers should keep it accurate.
		RebuildMountsSorted()
	end
	Refresh()
end)

local scrollbar = LibStub("tekKonfig-Scroll").new(FUR, 0, SCROLLSTEP)
local f = scrollbar:GetScript("OnValueChanged")
scrollbar:SetScript("OnValueChanged", function(self, value, ...)
	offset = math.floor(value)
	Refresh()
	if strfind((GetMouseFocus() and GetMouseFocus():GetName() or ""), "FURROW%d") then
		RowOnEnter(GetMouseFocus())
	end
	return f(self, value, ...)
end)


local offset = 0
FUR:EnableMouseWheel(true)
FUR:SetScript("OnMouseWheel", function(self, value)
	scrollbar:SetValue(scrollbar:GetValue() - value * SCROLLSTEP)
end)
FUR:SetScript("OnShow", function()
	scrollbar:SetMinMaxValues(0, math.max(0, GetNumCompanions(COMPANIONTYPE) - NUMROWS))
	scrollbar:SetValue(0)
	Refresh()
	if COMPANIONTYPE == "CRITTER" then
		sortbutton:Disable()
	else
		sortbutton:Enable()
	end
end)

-- Might want to check on this... var mounts? Learning new ones?
FUR:RegisterEvent("COMPANION_UPDATE")
FUR.COMPANION_UPDATE = function()
	Refresh();
end

local function Show()
	if FUR:IsShown() then
		Refresh()
	elseif PanelTemplates_GetSelectedTab(PetPaperDollFrame) and PanelTemplates_GetSelectedTab(PetPaperDollFrame) ~= 1 then
		if GetNumCompanions("CRITTER") == 0 then
			COMPANIONTYPE = "MOUNT"
		end
		FUR:Show()
	end
end
hooksecurefunc("PetPaperDollFrame_UpdateTabs", Show)

if PetPaperDollFrame:IsVisible() then Show() end -- This line runs only if the frame is open when the addon loads. o.0

-- "Pet" button
PetPaperDollFrameTab1:HookScript("OnClick", function(self)
	FUR:Hide();
end)

-- "Companions" button
PetPaperDollFrameTab2:HookScript("OnClick", function(self)
	COMPANIONTYPE = "CRITTER"
	sortbutton:Disable()
	scrollbar:SetMinMaxValues(0, math.max(0, GetNumCompanions(COMPANIONTYPE) - NUMROWS))
	scrollbar:SetValue(0)
	Show();
end)

-- "Mounts" button
PetPaperDollFrameTab3:HookScript("OnClick", function(self)
	COMPANIONTYPE = "MOUNT"
	sortbutton:Enable()
	scrollbar:SetMinMaxValues(0, math.max(0, GetNumCompanions(COMPANIONTYPE) - NUMROWS))
	scrollbar:SetValue(0)
	Show();
end)
